#include "B4RDefines.h"

namespace B4R {
	bool B4RRF95::Initialize(SubVoidByteArray DataAvailableSub, Byte ThisAddress) {
		this->DataAvailableSub = DataAvailableSub;
		FunctionUnion fu;
		fu.PollerFunction = looper;
		pnode.functionUnion = fu;
		pnode.tag = this;
		rf95 = new (backend) RH_RF95();
		rf95->setThisAddress(ThisAddress);
		rf95->setHeaderFrom(ThisAddress);
		bool r = rf95->init();
		if (r) {
			if (pnode.next == NULL)
				pollers.add(&pnode);
		}
		return r;
	}
	void B4RRF95::looper(void* b) {
		B4RRF95* me = (B4RRF95*)b;
		
		if (me->rf95->available()) {
			const UInt cp = B4R::StackMemory::cp;
			ArrayByte* arrPayload = CreateStackMemoryObject(ArrayByte);
			arrPayload->data = me->buffer;
			Byte len = RH_RF95_MAX_MESSAGE_LEN;
			me->rf95->recv(me->buffer, &len);
			arrPayload->length = len;
			me->DataAvailableSub(arrPayload);
			B4R::StackMemory::cp = cp;
		}
	}
	void B4RRF95::setModemConfig(RH_RF95::ModemConfigChoice mode) {
		rf95->setModemConfig(mode);
	}
	void B4RRF95::SetModemRegisters(byte Reg_1D, byte Reg_1E, Byte Reg26) {
		RH_RF95::ModemConfig mc;
		mc.reg_1d = Reg_1D;
		mc.reg_1e = Reg_1E;
		mc.reg_26 = Reg26;
		rf95->setModemRegisters(&mc);
	}
	bool B4RRF95::Send(Array* Data) {
		return rf95->send((Byte*)Data->data, Data->length);
	}
	UInt B4RRF95::getMaxMessageLength() {
		return RH_RF95_MAX_MESSAGE_LEN;
	}
	void B4RRF95::setFrequency(float Frequency) {
		rf95->setFrequency(Frequency);
	}
	void B4RRF95::TxPower(Byte Power, bool useRFO) {
		rf95->setTxPower(Power, useRFO);
	}
	bool B4RRF95::Sleep() {
		return rf95->sleep();
	}
	bool B4RRF95::isChannelActive() {
		return rf95->isChannelActive();
	}
	int B4RRF95::getLastSNR() {
		return rf95->lastSNR();
	}
	void B4RRF95::setHeaderTo(Byte To) {
		rf95->setHeaderTo(To);
	}
	Byte B4RRF95::getHeaderTo() {
		return rf95->headerTo();
	}
	void B4RRF95::setHeaderFrom(Byte From) {
		rf95->setHeaderFrom(From);
	}
	Byte B4RRF95::getHeaderFrom() {
		return rf95->headerFrom();
	}
	void B4RRF95::setHeaderID(Byte ID) {
		rf95->setHeaderId(ID);
	}
	Byte B4RRF95::getHeaderID() {
		return rf95->headerId();
	}
	void B4RRF95::setHeaderFlags(byte Set, byte Clear){
		rf95->setHeaderFlags(Set,Clear);
	}
	Byte B4RRF95::getHeaderFlags() {
		return rf95->headerFlags();
	}
}
